#include "sphere.h"

Sphere::Sphere(const Vector& c, double r) : center_(c), radius_(r) {    
}


const Vector& Sphere::getCenter() const { return center_; }

void Sphere::setCenter(const Vector& v) { center_ = v; }

double Sphere::getRadius() const { return radius_; }

void Sphere::setRadius(double r) { radius_ = r; }




inline double max(double x, double y) {
	return (x<y) ? y : x;
}
inline double min(double x, double y) {
	return (x>y) ? y : x;
}
    
bool Sphere::intersect(const Ray& ray,Vector& point,Vector& normal, double& distance) const {
	// ray-sphere intersection

	// ray direction
	const Vector direction(ray.getDirection());
	const double i = direction[0];
	const double j = direction[1];
	const double k = direction[2];

	// ray origin
	const Vector origin(ray.getOrigin());
	const double x = origin[0];
	const double y = origin[1];
	const double z = origin[2];

	// sphere's center
	const double l = center_[0];
	const double m = center_[1];
	const double n = center_[2];
	const double r = radius_;

	// compute parameters a,b,c for at^2+bt+c=0
	const double a = i*i + j*j + k*k;
	const double b = 2*(i*(x-l) + j*(y-m) + k*(z-n));
	const double c = l*l+m*m+n*n + x*x+y*y+z*z - 2*(l*x+m*y+n*z+r*r);

	// solve at^2+bt+c=0
	// D = b^2-4ac
	const double D = b*b-4*a*c;

	// no intersection
	if (D < 0)
		return false;

	// nearest ray intersection
	double t;

	if (D > 0)
	{
		// two intersections
		// ray parameter
		const double t1 = (-b + sqrt(D))/2*a;
		const double t2 = (-b - sqrt(D))/2*a;
		t = min(t1,t2);
		if (t < 0)
			t = max(t1,t2);
	}
	else
		// only one intersection
		t = -b/2*a;
		
	// all intersections behind the origin ?
	if (t <= 0.001) 
		return false;


	point  = ray.getOrigin() + t*ray.getDirection();
	normal = (point - center_)/r; // divison by "r" normalizes the vector
	distance = t;//abs(point-ray.getOrigin());

    return true;
}